=== modified file 'NEWS'
--- NEWS	2007-12-18 23:41:30 +0000
+++ NEWS	2007-12-19 02:44:29 +0000
@@ -43,6 +43,9 @@
    * Fixed Unicode encoding error in ``ignored`` when the output is
      redirected to a pipe. (Lukáš Lalinský)
 
+   * Fix traceback when sending large response bodies over the smart protocol
+     on Windows. (Andrew Bennetts, #115781)
+
    * Transform failures no longer cause ImmortalLimbo errors (Aaron Bentley,
      #137681)
 

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-12-18 15:22:47 +0000
+++ bzrlib/osutils.py	2007-12-19 02:44:29 +0000
@@ -1407,6 +1407,17 @@
     return b
 
 
+def send_all(socket, bytes):
+    """Send all bytes on a socket.
+
+    Regular socket.sendall() can give socket error 10053 on Windows.  This
+    implementation sends no more than 64k at a time, which avoids this problem.
+    """
+    chunk_size = 2**16
+    for pos in xrange(0, len(bytes), chunk_size):
+        socket.sendall(bytes[pos:pos+chunk_size])
+
+
 def dereference_path(path):
     """Determine the real path to a file.
 

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2007-12-03 16:39:11 +0000
+++ bzrlib/smart/medium.py	2007-12-19 02:25:26 +0000
@@ -30,6 +30,7 @@
 
 from bzrlib import (
     errors,
+    osutils,
     symbol_versioning,
     )
 from bzrlib.smart.protocol import (
@@ -177,7 +178,7 @@
         self.finished = True
 
     def _write_out(self, bytes):
-        self.socket.sendall(bytes)
+        osutils.send_all(self.socket, bytes)
 
 
 class SmartServerPipeStreamMedium(SmartServerStreamMedium):
@@ -524,7 +525,7 @@
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
-        self._socket.sendall(bytes)
+        osutils.send_all(self._socket, bytes)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""

